@@ -1,10 +1,12 @@
-__all__ = ['ResultException', 'ResultType', 'Result', 'Ok', 'Err', 'wrap_result']
+__all__ = ['ResultException', 'ResultType', 'Result', 'Ok', 'Err', 'wrap_result', 'auto_unwrap']
 
-import asyncio
 import types
+import asyncio
+from functools import wraps
 from typing import Any, Callable
 from inspect import iscoroutinefunction
 
+
 class ResultException(Exception):
     pass
 
@@ -74,14 +76,16 @@ class ResultType(_ResultType):
             E = cls.__annotations__['e']
             
             if asyncio.iscoroutinefunction(fn):
-                async def wrap(*args, **kwargs) -> Any:
+                # @wraps(fn)
+                async def wrap(*args, **kwargs) -> cls:
                     try:
                         v = await fn(*args, **kwargs)
                         return Ok[V](v)
                     except Exception as e:
                         return Err[E](e)
             elif callable(fn):
-                def wrap(*args, **kwargs) -> Any:
+                # @wraps(fn)
+                def wrap(*args, **kwargs) -> cls:
                     try:
                         v = fn(*args, **kwargs)
                         return Ok[V](v)
@@ -103,6 +107,22 @@ class Result(_Result):
         raise TypeError('Cannot be instantiated')
 
 
+    def __enter__(self):
+        try:
+            v = self.unwrap()
+        except Exception as e:
+            self.__exit__(type(e), e, e.__traceback__)
+
+        return v
+
+
+    def __exit__(self, exc_type, exc_value, traceback):
+        if exc_type or exc_value:
+            raise exc_value
+
+        return self
+
+
 class Ok(Result):
     __match_args__ = ('v',)
     v: Any
@@ -199,6 +219,7 @@ def wrap_result(res: Result):
     This function wraps the Result type and returns a Ok or Err type of Result
     """
     def outer(f):
+        @wraps(f)
         def inner(*args, **kwargs):
             if iscoroutinefunction(f):
                 async def a():
@@ -207,6 +228,7 @@ def wrap_result(res: Result):
                         return Ok(v)
                     except Exception as e:
                         return Err(e)  
+                
                 return a()
             else:
                 try:
@@ -219,3 +241,43 @@ def wrap_result(res: Result):
 
     return outer
 
+
+def auto_unwrap(obj) -> Any:
+    """
+    This function auto unwraps Ok or Err type of Result
+    """
+    if isinstance(obj, type):
+        type_ = obj
+        new_type_dict = {}
+
+        for a, f in type_.__dict__.items():
+            if not callable(f):
+                continue
+
+            return_type = f.__annotations__.get('return')
+
+            if not return_type:
+                continue  # pragma: no cover
+
+            if not issubclass(return_type, Result):
+                continue
+
+            new_f = auto_unwrap(f)
+            new_type_dict[a] = new_f
+
+        new_type = type(type_.__name__, (type_,), new_type_dict)
+        return new_type
+    else:
+        func = obj
+        
+        @wraps(func)
+        def wrapper(*args, **kwargs):
+            if iscoroutinefunction(obj):
+                async def a():
+                    return (await func(*args, **kwargs)).unwrap()
+                
+                return a()
+            else:
+                return func(*args, **kwargs).unwrap()
+
+    return wrapper
